# Copyright (C) 2011 Linaro Limited
#
# Author: Zygmunt Krynicki <zygmunt.krynicki@linaro.org>
#
# This file is part of django-debian.
#
# django-debian is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3
# as published by the Free Software Foundation
#
# django-debian is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with django-debian.  If not, see <http://www.gnu.org/licenses/>.

"""
Package with unit tests for django-debian
"""

import doctest
import unittest

TEST_MODULES = [
    'django_debian.config_file',
    'django_debian.dbconfig',
    'django_debian.secret_key',
    'django_debian.settings',
    'tests.test_config_file',
    'tests.test_fs_mock',
    'tests.test_settings',
]


def suite(modules=None):
    """
    Build an unittest.TestSuite() object with all the tests in _modules.
    Each module is harvested for both regular unittests and doctests
    """
    if modules is None:
        modules = TEST_MODULES
    suite = unittest.TestSuite()
    loader = unittest.TestLoader()
    for module_name in modules:
        try:
            module = __import__(module_name, fromlist=[''])
        except ImportError:
            import logging
            logging.exception("Unable to import test module %s", module_name)
            raise
        unit_suite = loader.loadTestsFromName(module_name)
        suite.addTests(unit_suite)
        doc_suite = doctest.DocTestSuite(module_name)
        suite.addTests(doc_suite)
    return suite



# Copyright (C) 2011 Linaro Limited
#
# Author: Zygmunt Krynicki <zygmunt.krynicki@linaro.org>
#
# This file is part of django-debian.
#
# django-debian is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3
# as published by the Free Software Foundation
#
# django-debian is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with django-debian.  If not, see <http://www.gnu.org/licenses/>.

"""
dbconfig-common integration for django
"""

import os

from django_debian.config_file import ConfigFile


def get_database(pathname):
    """
    Get django database settings for dbconfig-common file generated with
    dbconfig-generate-include in thg default shell mode.
    """
    config = ConfigFile.load(pathname)
    if config.dbtype == "sqlite3":
        ENGINE = "django.db.backends.sqlite3"
        NAME = os.path.join(config.basepath, config.dbname)
    elif config.dbtype == "mysql":
        ENGINE = "django.db.backends.mysql"
        NAME = config.dbname
    elif config.dbtype == "pgsql":
        ENGINE = "django.db.backends.postgresql_psycopg2"
        NAME = config.dbname
        # Work around for http://bugs.debian.org/cgi-bin/bugreport.cgi?bug=509627
        if config.dbserver == "":
            config.dbserver = "localhost"
    else:
        raise ValueError("Unupported value for dbtype: %r" % config.dbtype)
    return {
        'ENGINE': ENGINE,
        'NAME': NAME,
        'USER': config.dbuser,
        'PASSWORD': config.dbpass,
        'HOST': config.dbserver,
        'PORT': config.dbport}



# Copyright (C) 2011 Linaro Limited
#
# Author: Zygmunt Krynicki <zygmunt.krynicki@linaro.org>
#
# This file is part of django-debian.
#
# django-debian is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3
# as published by the Free Software Foundation
#
# django-debian is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with django-debian.  If not, see <http://www.gnu.org/licenses/>.

"""
Obtain or generate secret key on first use in Debian systems.
"""

from random import choice

from django_debian.config_file import ConfigFile


def _make_secret_key():
    """
    Generate a value that can be used as SECRET_KEY in Django settings
    """
    return ''.join([
        choice('abcdefghijklmnopqrstuvwxyz0123456789!@#$%^&*(-_=+)')
        for i in range(50)])


def _write_secret_key(pathname, secret_key):
    """
    Write a configuration file with the specified key
    """
    with open(pathname, 'wt') as stream:
        print >> stream, """
# This file was automatically generated by the django-debian package, upon
# first access, by the application that attempted to load it.  You can
# regenerate it at any time by removing this file or the SECRET_KEY variable
# defined below

# This key is used by Django to ensure the security of various cookies and
# one-time values. To learn more please visit:
# http://docs.djangoproject.com/en/1.2/ref/settings/#secret-key.

# Note: DO NOT PUBLISH THIS FILE.

SECRET_KEY='%s'
""" % secret_key


def get_secret_key(pathname):
    """
    Get Django secret key value from a file _or_ generate it on first use
    """
    try:
        return ConfigFile.load(pathname).SECRET_KEY
    except (AttributeError, IOError, ValueError) as ex:
        _write_secret_key(pathname, _make_secret_key())
    return ConfigFile.load(pathname).SECRET_KEY



# Copyright (C) 2011 Linaro Limited
#
# Author: Zygmunt Krynicki <zygmunt.krynicki@linaro.org>
#
# This file is part of django-debian.
#
# django-debian is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3
# as published by the Free Software Foundation
#
# django-debian is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with django-debian.  If not, see <http://www.gnu.org/licenses/>.

"""
Test suite for the package
"""

from unittest import TestCase

from django_debian.config_file import ConfigFile
from fs_mock import MockedFileSystem


class TestConfigFile(TestCase):

    _CONFIG_FILE_NAME = "config.conf"

    def mocked_config_file(self, contents):
        fs = MockedFileSystem()
        with fs.record():
            with open(self._CONFIG_FILE_NAME, "wt") as stream:
                stream.write(contents)
        with fs.replay():
            return ConfigFile.load(self._CONFIG_FILE_NAME)

    def test_load(self):
        data = self.mocked_config_file(
            "value1='foo'\n"
            "value2='bar'\n")
        self.assertEqual(data.value1, "foo")
        self.assertEqual(data.value2, "bar")



# Copyright (C) 2011 Linaro Limited
#
# Author: Zygmunt Krynicki <zygmunt.krynicki@linaro.org>
#
# This file is part of django-debian.
#
# django-debian is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3
# as published by the Free Software Foundation
#
# django-debian is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with django-debian.  If not, see <http://www.gnu.org/licenses/>.

"""
Tests for Settings class
"""

from unittest import TestCase
from fs_mock import MockedFileSystem

from django_debian.settings import Settings


class SettingsDefaultsTests(TestCase):

    def setUp(self):
        fs = MockedFileSystem()
        with fs.replay():
            # Replay in isolation mode acts as an empty filesystem
            # this is exactly what we want to observe.
            self.settings = Settings('test')

    def test_DEBUG(self):
        self.assertEqual(self.settings.DEBUG, False)

    def test_MEDIA_ROOT(self):
        self.assertEqual(self.settings.MEDIA_ROOT, "/var/lib/test/media/")

    def test_MEDIA_URL(self):
        self.assertEqual(self.settings.MEDIA_URL, "/test/media/")

    def test_STATIC_ROOT(self):
        self.assertEqual(self.settings.STATIC_ROOT, "/var/lib/test/static/")

    def test_STATIC_URL(self):
        self.assertEqual(self.settings.STATIC_URL, "/test/static/")

    def test_ADMIN_MEDIA_PREFIX(self):
        self.assertEqual(self.settings.ADMIN_MEDIA_PREFIX, "/test/static/admin/")

    def test_TEMPLATE_DIRS(self):
        self.assertEqual(self.settings.TEMPLATE_DIRS, (
            "/etc/test/templates/", "/usr/share/test/templates/"))

    def test_STATICFILES_DIRS(self):
        self.assertEqual(self.settings.STATICFILES_DIRS, (
            ("test", "/usr/share/test/htdocs/"),))

    def test_ADMINS(self):
        self.assertEqual(self.settings.ADMINS, (
            ("test Administrator", 'root@localhost'),))

    def test_MANAGERS(self):
        self.assertEqual(self.settings.ADMINS, (
            ("test Administrator", 'root@localhost'),))

    def test_SEND_BROKEN_LINK_EMAILS(self):
        self.assertEqual(self.settings.SEND_BROKEN_LINK_EMAILS, False)



# Copyright (C) 2011 Linaro Limited
#
# Author: Zygmunt Krynicki <zygmunt.krynicki@linaro.org>
#
# This file is part of django-debian.
#
# django-debian is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3
# as published by the Free Software Foundation
#
# django-debian is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with django-debian.  If not, see <http://www.gnu.org/licenses/>.

"""
Simple integration for Django settings.py
"""

from django_debian.secret_key import get_secret_key
from django_debian.dbconfig import get_database
import json
import os


class Settings(object):
    """
    Settings object for better integration with Django and Debian.

    Example usage (inside your settings.py):

        from django_debian.settings import Settings

        DEBIAN_SETTINGS = Settings("yourappname")
        DATABASES = {"default": DEBIAN_SETTINGS.default_database}
        SECRET_KEY = DEBIAN_SETTINGS.SECRET_KEY
        # Other settings

    A lot of the fields can be re-defined in a JSON configuration file.
    Usually they have good defaults that allows the application to work
    normally out-of-the-box.

    The settings file is constructed with the SETTINGS_TEMPLATE, the filename
    attribute is "settings" the appname attribute depends on the application.
    For example django-hello application the pathname would be:

        ``/etc/django-hello/settings.conf``

    Note: Some settings CANNOT be redefined using that file, in particular this includes:

    1. Default database settings (always loaded from dbconfig-common integration).
    2. SECRET_KEY value (stored in a separate, application-writable file).
    """

    # Template used to generate pathnames to various files this class uses
    SETTINGS_TEMPLATE = "/etc/{appname}/{filename}.conf"

    def __init__(self, appname, template=None):
        if template is None:
            template = os.environ.get("DJANGO_DEBIAN_SETTINGS_TEMPLATE", self.SETTINGS_TEMPLATE)
        self._settings_template = template
        self._appname = appname
        self._settings = self._load_settings()

    def _get_pathname(self, filename):
        """
        Calculate the pathname of the specified configuration file
        """
        return self._settings_template.format(appname=self._appname, filename=filename)

    def _load_settings(self):
        """
        Load and return settings from the "settings" configuration file.
        
        The file must be be using JSON format, it is simply loaded and used in
        other parts of this API.
        
        The file roughly represents the same format as would be created by
        serializing the settings.py module. Not all values are exposed though. 
        """
        pathname = self._get_pathname("settings")
        if os.path.exists(pathname):
            with open(pathname, "rt") as stream:
                return json.load(stream)
        else:
            return {}

    @property
    def DEBUG(self):
        """
        See: http://docs.djangoproject.com/en/1.2/ref/settings/#debug

        Bridge for the settings file DEBUG property.

        By default it produces the value ``False``

        Warning:
            Running production sites in with DEBUG is very dangerous. While in
            DEBUG mode the server will consume more memory with each incoming
            request as the history of all SQL commands is retained.
        """
        default = False 
        return self._settings.get("DEBUG", default)

    @property
    def SECRET_KEY(self):
        """
        See: http://docs.djangoproject.com/en/1.2/ref/settings/#secret-key

        The key is obtained from ``secret_key`` configuration file that may be
        overwritten on first use (it will be regenerated when corrupted or
        missing) 
        """
        pathname = self._get_pathname("secret_key")
        return get_secret_key(pathname)

    @property
    def default_database(self):
        """
        See: http://docs.djangoproject.com/en/1.2/ref/settings/#databases

        The returned value is suitable for the "default" database. The actual
        values are obtained from the "default_database" configuration file that
        is generated by dbconfig-common as requested by django-debian in one of
        the maintainer scripts
        """
        pathname = self._get_pathname("default_database")
        return get_database(pathname)

    @property
    def MEDIA_ROOT(self):
        """
        See: http://docs.djangoproject.com/en/1.2/ref/settings/#media-root

        Bridge for the settings file MEDIA_ROOT property.

        By default it produces the string:

            ``"/var/lib/{appname}/media/"``

        """
        default = "/var/lib/{appname}/media/".format(appname=self._appname)
        return self._settings.get("MEDIA_ROOT", default)

    @property
    def MEDIA_URL(self):
        """
        See: http://docs.djangoproject.com/en/1.2/ref/settings/#media-url

        Bridge for the settings file MEDIA_URL property.

        By default it produces the string:

            ``"/{appname}/media/"``

        """
        default = "/{appname}/media/".format(appname=self._appname)
        return self._settings.get("MEDIA_URL", default)

    @property
    def STATIC_ROOT(self):
        """
        Similar to MEDIA_ROOT but only for static files shipped with each application.

        Bridge for the settings file STATIC_ROOT property.

        By default it produces the string:

            ``"/var/lib/{appname}/static/"``

        """
        default = "/var/lib/{appname}/static/".format(appname=self._appname)
        return self._settings.get("STATIC_ROOT", default)

    @property
    def STATIC_URL(self):
        """
        Similar to MEDIA_URL but only for static files shipped with each application.

        Bridge for the settings file STATIC_URL property.

        By default it produces the string:

            ``"/{appname}/static/"``
        """
        default = "/{appname}/static/".format(appname=self._appname)
        return self._settings.get("STATIC_URL", default)

    @property
    def ADMIN_MEDIA_PREFIX(self):
        """
        See: http://docs.djangoproject.com/en/1.2/ref/settings/#admin-media-prefix

        Bridge for the settings file ADMIN_MEDIA_DIRS property.

        By default it produces the string:
            ``"{STATIC_URL}admin/"``
        """
        default = self.STATIC_URL + "admin/"
        return self._settings.get("ADMIN_MEDIA_PREFIX", default)

    @property
    def TEMPLATE_DIRS(self):
        """
        See: http://docs.djangoproject.com/en/1.2/ref/settings/#template-dirs

        Bridge for the settings file TEMPLATE_DIRS property.

        By default it produces two directories:

            * ``"/etc/{appname}/templates"``
            * ``"/usr/share/{appname}/templates"``

        The first one is provided for administrators that may wish to override
        one or more templates in a local installation for any purpose. The
        second one is to gain access to standard package templates.
        """
        default = (
            "/etc/{appname}/templates/".format(appname=self._appname),
            "/usr/share/{appname}/templates/".format(appname=self._appname))
        return self._settings.get("TEMPLATE_DIRS", default)

    @property
    def STATICFILES_DIRS(self):
        """
        Similar to TEMPLATE_DIRS but only for static files shipped with each application.

        Bridge for the settings file STATICFILES_DIRS property.

        By default it produces one directory:

            * ``"/usr/share/{appname}/htdocs/"``

        """
        default = (
            (self._appname, "/usr/share/{appname}/htdocs/".format(appname=self._appname)),
        )
        return self._settings.get("STATICFILES_DIRS", default)

    @property
    def ADMINS(self):
        """
        See: http://docs.djangoproject.com/en/1.2/ref/settings/#admins

        Bridge for the settings file ADMIN property.

        By default it produces only one value

            ``("{appname} Administrator", "root@localhost')``
        """
        default = (
            ('{appname} Administrator'.format(appname=self._appname), 'root@localhost'),
        )
        return self._settings.get("ADMINS", default) 

    @property
    def MANAGERS(self):
        """
        See: http://docs.djangoproject.com/en/1.2/ref/settings/#managers

        Bridge for the settings file MANAGERS property.

        By default it returns whatever ADMINS returns.
        """
        return self._settings.get("MANAGERS", self.ADMINS)

    @property
    def SEND_BROKEN_LINK_EMAILS(self):
        """
        See: http://docs.djangoproject.com/en/1.2/ref/settings/#send-broken-link-emails

        Bridge for the settings file SEND_BROKEN_LINK_EMAILS property.

        By default it returns False
        """
        default = False
        return self._settings.get("SEND_BROKEN_LINK_EMAILS", default)



# Copyright (C) 2011 Linaro Limited
#
# Author: Zygmunt Krynicki <zygmunt.krynicki@linaro.org>
#
# This file is part of django-debian.
#
# django-debian is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3
# as published by the Free Software Foundation
#
# django-debian is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with django-debian.  If not, see <http://www.gnu.org/licenses/>.

"""
File system mocking utilities for unit testing
"""

from StringIO import StringIO
from contextlib import contextmanager
import __builtin__
import os.path


class MockedIOError(IOError):
    """
    IOError subclass that is used by MockedFileSystem
    """

    def __init__(self, filename, mode, isolation_level, message):
        super(MockedIOError, self).__init__(filename, mode, isolation_level,
                                            message)
        self.filename = filename
        self.message = message
        self.mode = mode
        self.isolation_level = isolation_level

    def __str__(self):
        return ("Unable to open file %r in mode %r"
                " on in isolation level %r: %s" % (
                    self.filename, self.mode, self.isolation_level,
                    self.message))

    def __repr__(self):
        return ("MockedIOError(filename=%r, mode=%r,"
                " isolation_level=%r, message=%r)" % (
                    self.filename, self.mode, self.isolation_level,
                    self.message))


class StringIOWrapper(object):
    """
    Base class for using StringIO more like a normal file

    Currently it only provides the context management functions
    """

    def __init__(self, content):
        self._content = content

    def __iter__(self):
        """
        We need to directly forward __iter__ to the content object as object
        also has a definition of this function and thus __getattr__ will not
        work
        """
        return iter(self._content)

    def close(self):
        # XXX: Why doesn't __getattr__ pick this up?
        return self._content.close()

    def __getattr__(self, attr):
        return getattr(self._content, attr)

    def __enter__(self):
        return self

    def __exit__(self, *exc_info):
        self.close()


class MockedRecordedFile(StringIOWrapper):
    """
    Read-write file that is used during the recording stage of mocked
    filesystem usage.
    """

    def __init__(self, name):
        super(MockedRecordedFile, self).__init__(StringIO())
        self.name = name
        self._final_content = None

    def close(self):
        # We need to store that because StringIO.close() will discard the
        # internal buffer and StringIO.getvalue() will no longer work
        # afterwards.
        self._final_content = self._content.getvalue()
        return super(MockedRecordedFile, self).close()

    @property
    def final_content(self):
        """
        Get the final content of the mocked file.
        """
        if self.closed:
            return self._final_content
        else:
            return self.getvalue()


class MockedReplayedFile(StringIOWrapper):
    """
    Read only file that is used during the replay stage of mocked filesystem
    usage.
    """

    def __init__(self, name, content):
        super(MockedReplayedFile, self).__init__(StringIO(content))
        self.name = name

    def write(self, data):
        self._readonly_error()

    def writelines(self, lines):
        # XXX: Without this method unit tests break. Since writelines is
        # implemented on top of write and write raises IOError then this method
        # should also do that. For some odd reason it does not
        self._readonly_error()

    def truncate(self, size=None):
        self._readonly_error()

    def _readonly_error(self):
        raise IOError("File not open for writing")


class MockedFileSystem(object):
    """
    Helper class for mocking filesystem access.

    Currently it is limited to mocking access to files (directories are not
    supported). For files only file contents can be mocked, any permissions or
    other attributes are not supported.
    """

    FULL_ISOLATION = "full"
    PARTIAL_ISOLATION = "partial"

    def __init__(self):
        self._recorded_files = {}

    def _open_for_record(self, pathname, mode="r", buffering=0):
        if "w" in mode:
            if pathname not in self._recorded_files:
                recorded = MockedRecordedFile(pathname)
                self._recorded_files[pathname] = recorded
                return recorded
            else:
                raise MockedIOError(
                    pathname, mode, self._isolation_level,
                    "Attempt to open the same file for writing twice")
        else:
            if self._isolation_level == self.FULL_ISOLATION:
                raise MockedIOError(
                    pathname, mode, self._isolation_level,
                    "Reading files while recording in FULL_ISOLATION is"
                    "prohibited")
            elif self._isolation_level == self.PARTIAL_ISOLATION:
                return self._true_open(pathname, mode, buffering)

    def _open_for_replay(self, pathname, mode="r", buffering=0):
        if "w" in mode:
            raise MockedIOError(
                pathname, mode, self._isolation_level,
                "Writing to files in replaying in FULL_ISOLATION is"
                " prohibited")
        try:
            recorded = self._recorded_files[pathname]
            return MockedReplayedFile(recorded.name, recorded.final_content)
        except KeyError:
            if self._isolation_level == self.FULL_ISOLATION:
                raise MockedIOError(pathname, mode, self._isolation_level,
                                     "No previously recorded file found")
            elif self._isolation_level == self.PARTIAL_ISOLATION:
                return self._true_open(pathname, mode, buffering)

    def _os_path_exists(self, pathname):
        if self._isolation_level == self.FULL_ISOLATION:
            return pathname in self._recorded_files
        elif self._isolation_level == self.PARTIAL_ISOLATION:
            return (pathname in self._recorded_files or
                    self._true_os_path_exists(pathname))


    @contextmanager
    def record(self, isolation_level=FULL_ISOLATION):
        """
        Context manager for recording mocked files.

        Implemented by monkey-patching __builtin__.open

        Allows open() to be called for writing to arbitrary pathnames, open
        will return a file-like object (StringIO) that retains all the written
        data. The same data will be available when using replay().
        """
        # this is tricky, we _may_ get a non-true open but another overriden
        # value here but this is by design! It is used in unit tests to check
        # partial isolatin in two level mocking setup.
        self._true_open = open
        self._true_os_path_exists = os.path.exists
        self._isolation_level = isolation_level
        __builtin__.open = self._open_for_record
        __builtin__.file = self._open_for_record
        os.path.exists = self._os_path_exists
        yield
        __builtin__.open = self._true_open
        __builtin__.file = self._true_open
        os.path.exists = self._true_os_path_exists
        del self._true_open
        del self._true_os_path_exists
        del self._isolation_level

    @contextmanager
    def replay(self, isolation_level=FULL_ISOLATION):
        """
        Context manager for accessing previously recorded files.

        Implemented by monkey-patching __builtin__.open

        The optional isolation_level argument defines what happens when an
        attempt is made to access file that was _not_ recorded before. In
        FULL_ISOLATION an IOError with an appropriate error is raised. In
        PARTIAL_ISOLATION the call falls back to true open() with the same
        arguments.
        """
        # See the note in record() above
        self._true_open = open
        self._true_os_path_exists = os.path.exists
        self._isolation_level = isolation_level
        __builtin__.open = self._open_for_replay
        __builtin__.file = self._open_for_replay
        os.path.exists = self._os_path_exists
        yield
        __builtin__.open = self._true_open
        __builtin__.file = self._true_open
        os.path.exists = self._true_os_path_exists
        del self._true_open
        del self._true_os_path_exists
        del self._isolation_level



# Copyright (C) 2011 Linaro Limited
#
# Author: Zygmunt Krynicki <zygmunt.krynicki@linaro.org>
#
# This file is part of django-debian.
#
# django-debian is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3
# as published by the Free Software Foundation
#
# django-debian is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with django-debian.  If not, see <http://www.gnu.org/licenses/>.

"""
Tests for mocked file system utilities.
"""

from unittest import TestCase

from fs_mock import (
    MockedFileSystem,
    MockedIOError,
    MockedRecordedFile,
    MockedReplayedFile,
)
import os.path


class MockedRecordedFileTests(TestCase):

    _NAME = "name"
    _CONTENT = "content"

    def test_recorded_file_has_name(self):
        stream = MockedRecordedFile(self._NAME)
        self.assertEqual(stream.name, self._NAME)

    def test_final_content_works_after_closing(self):
        stream = MockedRecordedFile(self._NAME)
        stream.write(self._CONTENT)
        stream.close()
        self.assertEqual(stream.final_content, self._CONTENT)

    def test_final_content_works_before_closing(self):
        stream = MockedRecordedFile(self._NAME)
        stream.write(self._CONTENT)
        self.assertEqual(stream.final_content, self._CONTENT)

    def test_close_really_closes_file(self):
        stream = MockedRecordedFile(self._NAME)
        stream.close()
        self.assertTrue(stream.closed)


class MockedReplayedFileTests(TestCase):

    _NAME = "name"
    _CONTENT = "content"

    def setUp(self):
        self.stream = MockedReplayedFile(self._NAME, self._CONTENT)

    def test_write_is_prohibited(self):
        self.assertRaises(IOError, self.stream.write, "text")

    def test_writelines_is_prohibited(self):
        self.assertRaises(IOError, self.stream.writelines, ["text", ])

    def test_truncate_is_prohibited(self):
        self.assertRaises(IOError, self.stream.truncate)


class TestMockedFileSystem(TestCase):

    _NAME = "name"
    _CONTENT = "content"

    def setUp(self):
        self.fs = MockedFileSystem()

    def test_record_tracks_opened_files(self):
        with self.fs.record():
            open(self._NAME, "w").write(self._CONTENT)
        self.assertTrue(self._NAME in self.fs._recorded_files)

    def test_record_prevents_double_open(self):
        with self.fs.record():
            open(self._NAME, "w")
            self.assertRaises(MockedIOError, open, self._NAME, "w")

    def test_record_in_full_isolation_prohibits_accessing_non_mocked_files(
        self):
        with self.fs.record(MockedFileSystem.FULL_ISOLATION):
            self.assertRaises(MockedIOError, open, self._NAME)

    def test_replay_in_full_isolation_prohibits_accessing_non_moked_files(
        self):
        with self.fs.replay(MockedFileSystem.FULL_ISOLATION):
            self.assertRaises(MockedIOError, open, self._NAME)

    def test_record_in_parial_isolation_allows_accessing_existing_files(self):
        # Populate the outer filesystem with mocked file
        outer = MockedFileSystem()
        with outer.record():
            with open(self._NAME, "w") as stream:
                stream.write(self._CONTENT)
        with outer.replay():
            # Using the outer mocked filesystem use another mocked filesystem
            # to access files in partial isolation mode (it should fall-through
            # to the outer filesystem)
            with self.fs.record(MockedFileSystem.PARTIAL_ISOLATION):
                with open(self._NAME) as stream:
                    self.assertEqual(stream.read(), self._CONTENT)

    def test_recorded_files_exist_in_full_isolation_record(self):
        with self.fs.record(MockedFileSystem.FULL_ISOLATION):
            open(self._NAME, "w")
            self.assertTrue(os.path.exists, self._NAME)

    def test_recorded_files_exist_in_full_isolation_replay(self):
        with self.fs.record():
            open(self._NAME, "w")
        with self.fs.replay(MockedFileSystem.FULL_ISOLATION):
            self.assertTrue(os.path.exists, self._NAME)
            
    def test_record_in_partial_isolation_allows_checking_for_existing_files(self):
        outer = MockedFileSystem()
        with outer.record():
            open(self._NAME, "w")
        with outer.replay():
            with self.fs.record(MockedFileSystem.PARTIAL_ISOLATION):
                self.assertTrue(os.path.exists, self._NAME)
            
    def test_replay_in_partial_isolation_allows_checking_for_existing_files(self):
        outer = MockedFileSystem()
        with outer.record():
            open(self._NAME, "w")
        with outer.replay():
            with self.fs.replay(MockedFileSystem.PARTIAL_ISOLATION):
                self.assertTrue(os.path.exists, self._NAME)

    def test_replay_in_partial_isolation_allows_accessing_existing_files(self):
        # Populate the outer filesystem with mocked file
        outer = MockedFileSystem()
        with outer.record():
            with open(self._NAME, "w") as stream:
                stream.write(self._CONTENT)
        with outer.replay():
            # Using the outer mocked filesystem use another mocked filesystem
            # to access files in partial isolation mode (it should fall-through
            # to the outer filesystem)
            with self.fs.replay(MockedFileSystem.PARTIAL_ISOLATION):
                with open(self._NAME) as stream:
                    self.assertEqual(stream.read(), self._CONTENT)



# Copyright (C) 2011 Linaro Limited
#
# Author: Zygmunt Krynicki <zygmunt.krynicki@linaro.org>
#
# This file is part of django-debian.
#
# django-debian is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3
# as published by the Free Software Foundation
#
# django-debian is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with django-debian.  If not, see <http://www.gnu.org/licenses/>.

"""
Simple shell-sourcable configuration file class
"""

import re


class ConfigFile(object):
    """
    Configuration file parser compatible with files generated
    dbconfig-generate-include using sh format.
    """

    _pattern = re.compile(
        "^(?P<key>[_a-zA-Z][_a-zA-Z0-9]*)='(?P<value>[^']*)'\s*(?:#.*)?$")

    @classmethod
    def load(cls, pathname):
        """
        Load file from pathname and store all the values as instance attributes
        """
        self = cls()
        for key, value in cls._parse(pathname):
            setattr(self, key, value)
        return self

    @classmethod
    def _parse(cls, pathname):
        """
        Parse the contents of pathname and return key-value pairs
        """
        with open(pathname, "rt") as stream:
            for lineno, line in enumerate(stream, start=1):
                match = cls._pattern.match(line)
                if match:
                    yield match.group("key"), match.group("value")



#!/usr/bin/python
# Copyright (C) 2011 Linaro Limited
#
# Author: Zygmunt Krynicki <zygmunt.krynicki@linaro.org>
#
# This file is part of django-debian.
#
# django-debian is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3
# as published by the Free Software Foundation
#
# django-debian is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with django-debian.  If not, see <http://www.gnu.org/licenses/>.

from setuptools import setup, find_packages


setup(
    name="django-debian",
    version=":versiontools:django_debian:",
    description='Debian integration for Django applications',
    author='Zygmunt Krynicki',
    author_email='zygmunt.krynicki@canonical.com',
    url='http://launchpad.net/django-debian',
    test_suite='tests.suite',
    long_description="""
    Creating proper Debian packages for even most simple Django
    applications is not easy.

    This package provides integration for the following aspcts:

    * loading database settings from config file generated with dbconfig-common
    * loading and generating SECRET_KEY
    * integration with django-staticfiles (via maintainer scripts)
    * initial placement and configuration of important directories (via maintainer scripts)
    """,
    packages=find_packages(exclude=['tests']),
    install_requires=["django-seatbelt >= 1.0"],
    setup_requires=["versiontools >= 1.4"],
    data_files=[
        ('share/django-debian/dpkg/', [
            'dpkg/common',
            'dpkg/config',
            'dpkg/postinst',
            'dpkg/postrm',
            'dpkg/prerm',
            'dpkg/rules']),
        ('share/django-debian/dpkg/webserver/', [
            'dpkg/webserver/apache2',
            'dpkg/webserver/none']),
    ],
    license="LGPL3",
    zip_safe=True,
)
